-- module_crib.lua: Ultor Crib Triggers [nclok1405]

-- Triggers
CRIB_TRIGGERS = {
	-- Stores
	{prerequisite = "mIntro", x = -335.10, y = 0.19, z = -215.15, dist = 4, func = "crib_open_weapon", label = "MAP_ICON_WEAPON"},
	{prerequisite = "mIntro", x = -335.22, y = 0.19, z = -182.93, dist = 4, func = "crib_open_clothing", label = "MAP_ICON_CLOTHING"},
	{prerequisite = "mIntro", x = -314.46, y = 0.19, z = -189.97, dist = 4, func = "crib_open_tattoo", label = "MAP_ICON_TATTOO_PARLOR"},
	{prerequisite = "mIntro", x = -314.11, y = 0.19, z = -207.94, dist = 4, func = "crib_open_character", label = "MAP_ICON_PLASTIC_SURGEON"},

	-- Ultor HQ Elevator
	{prerequisite = "mIntro", x = -308.04, y = 0.12, z = -199.03, dist = 1.5, func = "crib_enter_training_area", label = "SUI_ELEVATOR_UP"},
	{prerequisite = "mIntro", x = 187.72, y = 942.02, z = 782.14, dist = 1.5, func = "crib_leave_training_area", label = "SUI_ELEVATOR_DOWN"},

	-- Vlad Prison
	{prerequisite = "mIntroVlad", x = 1352.30, y = 19.50, z = -703.46, dist = 4, func = "crib_enter_prison_area", label = "GM_EVENT_INT_VLAD"},
	{prerequisite = "mIntroVlad", x = -965.20, y = 951.35, z = 969.72, dist = 4, func = "crib_leave_prison_area", label = "HUD_OPEN_DOOR_MESSAGE"},

	-- Satan Fight Arena (OOB)
	{prerequisite = "mSatan", x = 233.27, y = 153.53, z = -224.14, dist = 1.1, func = "crib_enter_satan_area", label = "HUD_OPEN_DOOR_MESSAGE"},
	--{prerequisite = "mSatan", x = 2497.74, y = 0.00, z = -1949.93, dist = 1.5, func = "crib_leave_satan_area", label = "HUD_OPEN_DOOR_MESSAGE"},
}

-- Mission List
-- mission_name: Internal Mission ID
-- display_name: Displayed Mission Name
CRIB_MISSIONS = {
	{ mission_name = "mIntro", display_name = "GM_INTRO_TITLE" },
	{ mission_name = "mFlightTutorial", display_name = "SRG_MISSION_NAME_FLIGHT_TUT" },
	{ mission_name = "mIntroBB", display_name = "SRG_MISSION_NAME_INTROBB" },
	{ mission_name = "mIntroShakes", display_name = "SRG_MISSION_NAME_INTROSHAKES" },
	{ mission_name = "mIntroTwins", display_name = "SRG_MISSION_NAME_INTROTWINS" },
	{ mission_name = "mIntroVlad", display_name = "SRG_MISSION_NAME_INTROVLAD" },
	{ mission_name = "mmSummonTutorial", display_name = "GM_EVENT_TUT_SUMMON" },
	{ mission_name = "mmStompTutorial", display_name = "GM_EVENT_TUT_STOMP" },
	{ mission_name = "mmBlastTutorial", display_name = "GM_EVENT_TUT_BLAST" },
	{ mission_name = "mmAuraTutorial", display_name = "GM_EVENT_TUT_AURA" },
	{ mission_name = "mSatan", display_name = "SRG_MISSION_NAME_SATAN" },

	{ mission_name = "extraction", display_name = "MAP_ICON_EXTRACTION" },
}

-- Previous Buttons
CRIB_PREV_BUTTONS = {
	"CBA_VDC_PREV_RADIO",
	"CBA_OFC_TAUNT_TWO",
	"CBA_GAC_WEAPON_SELECT_1",
	"CBA_OFC_CROUCH",
	"CBA_OFC_CROUCH_TOGGLE",
}
-- Next Buttons
CRIB_NEXT_BUTTONS = {
	"CBA_VDC_NEXT_RADIO",
	"CBA_OFC_TAUNT_THREE",
	"CBA_GAC_WEAPON_SELECT_2",
	"CBA_OFC_PICKUP_RELOAD",
}

-- Mission Replay position
CRIB_MR_X = -337.28 -- -308.04
CRIB_MR_Y = 0.20 -- 0.12
CRIB_MR_Z = -198.61 -- -199.03
CRIB_MR_DIST = 1.5

Crib_mission_table = {}	-- List of selectable missions
Crib_selected_mission_number = 1	-- Currently selected mission number
Crib_mr_prompt_active = false	-- Mission Replay prompt active
Crib_mr_help_displayed = false	-- Mission Replay Help Displayed

-- Main Thread
-- Run it by `thread_new("crib_main_thread")` from sr4_5_city_main()
function crib_main_thread()
	fade_in_block()	-- Wait until the loading screen goes away

	-- Wait until the player completes the prologue
	while (not mission_is_complete("mFlightTutorial")) or mission_is_active() do
		thread_yield()
	end

	local status = {}
	local dismiss_timer = 0

	crib_update_mission_list()	-- Setup the completed mission list

	while true do
		thread_yield()

		-- If mission becomes active, disable everything and wait until it's over
		if mission_is_active() then
			for player_idx, player in pairs(player_names_get_all()) do
				hud_prompt_clear(player)
			end

			-- Wait until the mission is no longer active
			while mission_is_active() do
				thread_yield()
			end
			
			-- Setup the completed mission list again, so when a new mission is completed it is added to the replay
			crib_update_mission_list()
			
			party_allow_max_followers(true)	-- (Re)enable Homies
			tutorial_lock("recruiting")	-- No tutorial popup for recruiting (Z key doesn't work)
		end

		-- Check crib triggers for each player
		for player_idx, player in pairs(player_names_get_all()) do
			if status[player] == nil then
				status[player] = {}	-- Create a status table for this player
			end

			for trigger_idx, trigger in pairs(CRIB_TRIGGERS) do
				if (not status[player][trigger_idx]) and crib_is_close_enough(player, trigger.x, trigger.y, trigger.z, trigger.dist) and mission_is_complete(trigger.prerequisite) then
					-- Prompt is now visible
					status[player][trigger_idx] = true
					
					local funcname = trigger.func
					if player == LOCAL_PLAYER then
						funcname = funcname .. "_local"
					else
						funcname = funcname .. "_remote"
					end
					hud_prompt(player, trigger.label, funcname)
				elseif status[player][trigger_idx] and (not crib_is_close_enough(player, trigger.x, trigger.y, trigger.z, trigger.dist)) then
					-- Player left the prompt area
					status[player][trigger_idx] = false
					hud_prompt_clear(player)
				end
			end
		end

		-- Mission Replay
		if not mission_is_active() then
			if (not Crib_mr_prompt_active) and crib_is_close_enough(LOCAL_PLAYER, CRIB_MR_X, CRIB_MR_Y, CRIB_MR_Z, CRIB_MR_DIST) then
				Crib_mr_prompt_active = true
				hud_prompt(LOCAL_PLAYER, Crib_mission_table[Crib_selected_mission_number].display_name, "crib_mr_triggered")
				
				if not Crib_mr_help_displayed then
					message("SUI_CHOOSE_WITH_CROUCH_OR_RELOAD", 5, true, SYNC_LOCAL)
					Crib_mr_help_displayed = true
				end
			elseif Crib_mr_prompt_active and (not crib_is_close_enough(LOCAL_PLAYER, CRIB_MR_X, CRIB_MR_Y, CRIB_MR_Z, CRIB_MR_DIST)) then
				Crib_mr_prompt_active = false
				hud_prompt_clear(LOCAL_PLAYER)
			end

			-- Next
			if Crib_mr_prompt_active and crib_is_next_button_pressed() then
				audio_object_post_event("UI_STAG_PROTOTYPE_SCROLL", nil, nil, LOCAL_PLAYER)
				hud_prompt_clear(LOCAL_PLAYER)
				Crib_selected_mission_number = Crib_selected_mission_number + 1
				if Crib_selected_mission_number > #Crib_mission_table then
					Crib_selected_mission_number = 1
				end
				hud_prompt(LOCAL_PLAYER, Crib_mission_table[Crib_selected_mission_number].display_name, "crib_mr_triggered")
			end
			-- Previous
			if Crib_mr_prompt_active and crib_is_prev_button_pressed() then
				audio_object_post_event("UI_STAG_PROTOTYPE_SCROLL", nil, nil, LOCAL_PLAYER)
				hud_prompt_clear(LOCAL_PLAYER)
				Crib_selected_mission_number = Crib_selected_mission_number - 1
				if Crib_selected_mission_number < 1 then
					Crib_selected_mission_number = #Crib_mission_table
				end
				hud_prompt(LOCAL_PLAYER, Crib_mission_table[Crib_selected_mission_number].display_name, "crib_mr_triggered")
			end
		end

		-- Z: Dismiss Homies
		if (not mission_is_active()) and (crib_get_party_count() >= 1) and player_action_is_pressed("CBA_GAC_RECRUIT_DISMISS") then
			if dismiss_timer == 0 then
				message_remove_all()
				message("CONTROL_DESC_FOOT_DISMISS", 2, true, SYNC_LOCAL)
			end
			
			dismiss_timer = dismiss_timer + get_frame_time()
			
			if dismiss_timer >= 2 then
				message_remove_all()
				party_dismiss_all()
				dismiss_timer = 0
			end
		elseif dismiss_timer > 0 then
			dismiss_timer = 0
			message_remove_all()
		end
		
	end
end

-- Next button pressed
function crib_is_next_button_pressed()
	local next_button_pressed = false
	for i, action in pairs(CRIB_NEXT_BUTTONS) do
		if player_action_just_pressed(action) then
			next_button_pressed = true
			break
		end
	end
	return next_button_pressed
end

-- Prev button pressed
function crib_is_prev_button_pressed()
	local prev_button_pressed = false
	for i, action in pairs(CRIB_PREV_BUTTONS) do
		if player_action_just_pressed(action) then
			prev_button_pressed = true
			break
		end
	end
	return prev_button_pressed
end

-- Setup the mission list
function crib_update_mission_list()
	Crib_mission_table = {}
	for i, mission in pairs(CRIB_MISSIONS) do
		if mission.mission_name == "extraction" or mission_is_complete(mission.mission_name) then
			Crib_mission_table[#Crib_mission_table + 1] = mission
		end
	end
end

-- Mission Replay Trigger
function crib_mr_triggered()
	Crib_mr_prompt_active = false
	hud_prompt_clear(LOCAL_PLAYER)

	local id = Crib_mission_table[Crib_selected_mission_number].mission_name

	if id == "extraction" then
		local dialog_choice = open_vint_dialog("SUI_XT_RESET_TITLE", "SUI_XT_RESET_BODY", "CONTROL_YES", "CONTROL_NO")
		
		if dialog_choice == 0 then
			audio_object_post_event("UI_HUD_SURVIVAL_WAVE_START", nil, nil, LOCAL_PLAYER)
			extraction_reset("XT_ROOT_ne")
			extraction_reset("XT_ROOT_nw")
			extraction_reset("XT_ROOT_se")
			extraction_reset("XT_ROOT_sw")
			message_remove_all()
			message("SUI_XT_RESET_DONE", 5, true, SYNC_ALL)
		end
	else
		audio_object_post_event("UI_HUD_SURVIVAL_WAVE_START", nil, nil, LOCAL_PLAYER)
		mission_set_next_mission(id)
	end
end

-- Stores
function crib_open_weapon_local()
	hud_push_screen("store_common", SYNC_LOCAL)
	hud_push_screen("store_weapon", SYNC_LOCAL)
end
function crib_open_clothing_local()
	hud_push_screen("store_common", SYNC_LOCAL)
	hud_push_screen("store_clothing", SYNC_LOCAL)
end
function crib_open_tattoo_local()
	hud_push_screen("store_common", SYNC_LOCAL)
	hud_push_screen("store_tattoo", SYNC_LOCAL)
end
function crib_open_character_local()
	hud_push_screen("store_common", SYNC_LOCAL)
	hud_push_screen("store_character", SYNC_LOCAL)
end

function crib_open_weapon_remote()
	hud_push_screen("store_common", SYNC_REMOTE)
	hud_push_screen("store_weapon", SYNC_REMOTE)
end
function crib_open_clothing_remote()
	hud_push_screen("store_common", SYNC_REMOTE)
	hud_push_screen("store_clothing", SYNC_REMOTE)
end
function crib_open_tattoo_remote()
	hud_push_screen("store_common", SYNC_REMOTE)
	hud_push_screen("store_tattoo", SYNC_REMOTE)
end
function crib_open_character_remote()
	hud_push_screen("store_common", SYNC_REMOTE)
	hud_push_screen("store_character", SYNC_REMOTE)
end

-- Ultor HQ Elevator
function crib_enter_training_area_local()
	city_zone_swap("u_train", true)	-- Load Ultor HQ zone
	crib_teleport(LOCAL_PLAYER, 187.72, 942.02, 782.14)
end
function crib_enter_training_area_remote()
	city_zone_swap("u_train", true)	-- Load Ultor HQ zone
	crib_teleport(REMOTE_PLAYER, 187.72, 942.02, 782.14)
end

function crib_leave_training_area_local()
	crib_teleport(LOCAL_PLAYER, -308.04, 0.12, -199.03)
	if coop_is_active() then
		crib_teleport(REMOTE_PLAYER, -308.04, 0.12, -199.03)
	end
	city_zone_swap("u_train", false, true)	-- Unload Ultor HQ zone
end
function crib_leave_training_area_remote()
	crib_leave_training_area_local()
end

-- Vlad Prison
function crib_enter_prison_area_local()
	city_zone_swap("m_prison", true)	-- Load Prison zone
	crib_teleport(LOCAL_PLAYER, -965.20, 951.35, 969.72)
end
function crib_enter_prison_area_remote()
	city_zone_swap("m_prison", true)	-- Load Prison zone
	crib_teleport(REMOTE_PLAYER, -965.20, 951.35, 969.72)
end

function crib_leave_prison_area_local()
	crib_teleport(LOCAL_PLAYER, 1352.30, 19.50, -703.46)
	if coop_is_active() then
		crib_teleport(REMOTE_PLAYER, 1352.30, 19.50, -703.46)
	end
	city_zone_swap("m_prison", false)	-- Unload Prison zone
end
function crib_leave_prison_area_remote()
	crib_leave_prison_area_local()
end

-- Satan Fight Arena (Out of Bounds, but can still somewhat be explored by jumping repeatedly)
function crib_enter_satan_area_local()
	city_zone_swap("m_Sthrone", true)	-- Load final battle zone
	crib_teleport(LOCAL_PLAYER, 2497.74, 0.00, -1949.93)
	if coop_is_active() then
		crib_teleport(REMOTE_PLAYER, 2497.74, 0.00, -1949.93)
	end

	-- Leave when the player pushes next/prev buttons
	while true do
		thread_yield()
		
		if crib_is_prev_button_pressed() or crib_is_next_button_pressed() then
			crib_leave_satan_area_local()
			break
		end
	end
end
function crib_enter_satan_area_remote()
	crib_enter_satan_area_local()
end

function crib_leave_satan_area_local()
	crib_teleport(LOCAL_PLAYER, 233.27, 153.53, -224.14)
	if coop_is_active() then
		crib_teleport(REMOTE_PLAYER, 233.27, 153.53, -224.14)
	end
	city_zone_swap("m_Sthrone", false)	-- Unload final battle zone
end
function crib_leave_satan_area_remote()
	crib_leave_satan_area_local()
end

-- Check if player's position is close enough to the specified location
function crib_is_close_enough(player, ox, oy, oz, dist)
	local px, py, pz = get_object_pos(player)

	local dx = px - ox
	local dy = py - oy
	local dz = pz - oz

	if dx < 0 then
		dx = -dx
	end
	if dy < 0 then
		dy = -dy
	end
	if dz < 0 then
		dz = -dz
	end

	if (dx < dist) and (dy < dist) and (dz < dist) then
		return true
	end
	return false
end

-- Get party count
function crib_get_party_count()
	local count = 0
	if not character_is_dead("#FOLLOWER1#") then
		count = count + 1
	end
	if not character_is_dead("#FOLLOWER2#") then
		count = count + 1
	end
	if not character_is_dead("#FOLLOWER3#") then
		count = count + 1
	end
	return count
end

-- Teleport the specified player to the specified location
function crib_teleport(player, x, y, z)
	-- Used for custom teleports
	local stele_Posx = -358.60131835938
	local stele_Posy = 0.12552392482758
	local stele_Posz = -224.81632995605
	local obj_name = "mIntroBB"

	local exit_vehicle = false
	local reset_camera_orient = false
	local offset_in_local_spc = false
	local force_to_navmesh = false
	local heading_offset = 0.0
	local navmesh_hint_obj = obj_name

	teleport_to_object(player, obj_name, exit_vehicle, reset_camera_orient, x - stele_Posx, y - stele_Posy, z - stele_Posz, offset_in_local_spc, heading_offset, force_to_navmesh, navmesh_hint_obj)
end
